"use strict";
require('reflect-metadata');
var IoC_1 = require('../core/IoC');
var IoCSymbols_1 = require('../core/IoCSymbols');
var ControllerRegistration_1 = require('../models/ControllerRegistration');
/**
 * Controller decorator; decorates a class to be a rest api controller. A controller registers itself to an
 * expressJS router when "registerControllers" is called.
 *
 * @param {string} [routePrefix] - Prefix for the whole controller. This path is added to all routes.
 * @param {RequestHandler[]} [middlewares] - Middleware functions for the controller to be executed before the routing functions.
 * @returns {(Function) => void} - Decorator for the controller class.
 */
function Controller(routePrefix) {
    var middlewares = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        middlewares[_i - 1] = arguments[_i];
    }
    return function (controller) {
        IoC_1.IocContainer.get(IoCSymbols_1.IoCSymbols.registrar).addController(new ControllerRegistration_1.ControllerRegistration(controller, routePrefix, middlewares));
    };
}
exports.Controller = Controller;
