"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var Errors_1 = require('../errors/Errors');
var ParamDecorators_1 = require('../params/ParamDecorators');
var inversify_1 = require('inversify');
var PRIMITIVE_TYPES = [Object, String, Array, Number, Boolean];
var bodyParserInstalled;
try {
    require('body-parser');
    bodyParserInstalled = true;
}
catch (e) {
    bodyParserInstalled = false;
}
var CookieHelper = (function () {
    function CookieHelper(value) {
        var split = value.split('=');
        this.name = split[0];
        this.value = split[1];
    }
    return CookieHelper;
}());
var DefaultParamHandler = (function () {
    function DefaultParamHandler() {
    }
    DefaultParamHandler.prototype.extractParam = function (request, param) {
        switch (param.paramType) {
            case ParamDecorators_1.ParamType.Body:
                return request.body;
            case ParamDecorators_1.ParamType.Url:
                return request.params[param.name];
            case ParamDecorators_1.ParamType.Query:
                var options = param.options;
                if (!options || !options.alias) {
                    return request.query[param.name];
                }
                var aliases = !Array.isArray(options.alias) ? [options.alias] : options.alias;
                aliases = aliases.map(function (a) { return request.query[a]; }).filter(Boolean);
                return aliases[0] || request.query[param.name];
            case ParamDecorators_1.ParamType.Header:
                return request.get(param.name);
            case ParamDecorators_1.ParamType.Cookie:
                var cookies = request.get('cookie');
                if (!cookies) {
                    return undefined;
                }
                var cookie = cookies
                    .split(';')
                    .map(function (o) { return new CookieHelper(o.trim()); })
                    .filter(function (o) { return o.name === param.name; })[0];
                return cookie ? cookie.value : undefined;
        }
    };
    DefaultParamHandler.prototype.parseParam = function (rawValue, param) {
        var factory;
        if (param.options && param.options.factory) {
            factory = param.options.factory;
        }
        else {
            factory = function (raw) {
                var ctor = param.type;
                if (raw.constructor === ctor) {
                    return raw;
                }
                else {
                    return PRIMITIVE_TYPES.indexOf(ctor) !== -1 ? ctor(raw) : new ctor(raw);
                }
            };
        }
        if ((rawValue === null || rawValue === undefined) && param.options && param.options.required) {
            throw new Errors_1.RequiredParameterNotProvidedError(param.name);
        }
        else if (rawValue === null || rawValue === undefined) {
            return undefined;
        }
        var parsed;
        try {
            parsed = factory(rawValue);
        }
        catch (e) {
            throw new Errors_1.ParameterParseError(param.name, e);
        }
        if (param.options && param.options.validator) {
            var isValid = function (value) {
                var predicates = param.options.validator;
                if (Array.isArray(predicates)) {
                    return predicates.every(function (p) { return p(value); });
                }
                return predicates(value);
            };
            if (isValid(parsed)) {
                return parsed;
            }
            throw new Errors_1.ParamValidationFailedError(param.name);
        }
        else {
            return parsed;
        }
    };
    DefaultParamHandler.prototype.getParamValuesForRequest = function (params, request, response) {
        var _this = this;
        var paramValues = [];
        params.forEach(function (p) {
            switch (p.paramType) {
                case ParamDecorators_1.ParamType.Request:
                    paramValues[p.index] = request;
                    return;
                case ParamDecorators_1.ParamType.Response:
                    paramValues[p.index] = response;
                    return;
                default:
                    paramValues[p.index] = _this.parseParam(_this.extractParam(request, p), p);
                    return;
            }
        });
        return paramValues;
    };
    DefaultParamHandler.prototype.getParamsForRoute = function (target, routeKey) {
        var params = Reflect.getOwnMetadata(ParamDecorators_1.PARAMS_KEY, target, routeKey) || [];
        if (params.some(function (p) { return p.paramType === ParamDecorators_1.ParamType.Body; }) && !bodyParserInstalled) {
            console.warn("A route uses a @Body parameter, but there is no 'body-parser' package installed.");
        }
        params.forEach(function (p) {
            if (p.type.length < 1 && !(p.options && p.options.factory)) {
                throw new Errors_1.ParameterConstructorArgumentsError(p.name);
            }
        });
        return params;
    };
    DefaultParamHandler = __decorate([
        inversify_1.injectable(), 
        __metadata('design:paramtypes', [])
    ], DefaultParamHandler);
    return DefaultParamHandler;
}());
exports.DefaultParamHandler = DefaultParamHandler;
