"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
require('reflect-metadata');
var IoCSymbols_1 = require('./IoCSymbols');
var express_1 = require('express');
var inversify_1 = require('inversify');
/**
 * Default implementation of the registrar class.
 * Handles the registration of controllers.
 *
 * @class
 */
var DefaultRegistrar = (function () {
    function DefaultRegistrar(routeHandler) {
        this.routeHandler = routeHandler;
        this.controllers = [];
    }
    DefaultRegistrar.prototype.registerControllersFromFolder = function (_a, baseUrl, router) {
        var _this = this;
        var folderPath = _a.folderPath, _b = _a.root, root = _b === void 0 ? process.cwd() : _b, _c = _a.recursive, recursive = _c === void 0 ? false : _c, _d = _a.matchRegExp, matchRegExp = _d === void 0 ? /(.*)[.]js$/ : _d;
        if (baseUrl === void 0) { baseUrl = ''; }
        if (router === void 0) { router = express_1.Router(); }
        var filewalker = require('filewalker'), path = require('path');
        return new Promise(function (resolve, reject) {
            var controllersPath = path.join(root, folderPath);
            console.info("Load controllers from path '" + controllersPath + "' " + (recursive ? '' : 'non ') + "recursive.");
            filewalker(controllersPath, { recursive: recursive, matchRegExp: matchRegExp })
                .on('error', function (err) {
                console.error("Error happened during loading controllers from path: " + err);
                reject(err);
            })
                .on('file', function (controller) {
                try {
                    console.info("Loading controller '" + controller + "'.");
                    require(path.join(controllersPath, controller));
                }
                catch (e) {
                    console.error("Error happened during load of the '" + controller + "' controller: " + e);
                    reject(e);
                }
            })
                .on('done', function () {
                resolve(_this.registerControllers(baseUrl, router));
            })
                .walk();
        });
    };
    DefaultRegistrar.prototype.registerControllers = function (baseUrl, router) {
        if (baseUrl === void 0) { baseUrl = ''; }
        if (router === void 0) { router = express_1.Router(); }
        var url = baseUrl;
        if (url[url.length - 1] !== '/') {
            url += '/';
        }
        if (url[0] !== '/') {
            url = '/' + url;
        }
        for (var _i = 0, _a = this.controllers; _i < _a.length; _i++) {
            var ctrl = _a[_i];
            this.routeHandler.addRoutes(ctrl, url);
        }
        return this.routeHandler.registerRoutes(router);
    };
    DefaultRegistrar.prototype.addController = function (controller) {
        this.controllers.push(controller);
    };
    DefaultRegistrar.prototype.resetControllerRegistrations = function () {
        this.routeHandler.resetRoutes();
        this.controllers = [];
    };
    DefaultRegistrar = __decorate([
        inversify_1.injectable(),
        __param(0, inversify_1.inject(IoCSymbols_1.IoCSymbols.routeHandler)), 
        __metadata('design:paramtypes', [Object])
    ], DefaultRegistrar);
    return DefaultRegistrar;
}());
exports.DefaultRegistrar = DefaultRegistrar;
