"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var ControllerErrorHandler_1 = require('../errors/ControllerErrorHandler');
var ErrorHandlerDecorator_1 = require('../errors/ErrorHandlerDecorator');
var Errors_1 = require('../errors/Errors');
var VersionInformation_1 = require('../models/VersionInformation');
var ParamDecorators_1 = require('../params/ParamDecorators');
var RouteDecorators_1 = require('../routes/RouteDecorators');
var RouteHelpers_1 = require('../utilities/RouteHelpers');
var VersionDecorator_1 = require('../versioning/VersionDecorator');
var IoCSymbols_1 = require('./IoCSymbols');
var express_1 = require('express');
var httpStatus = require('http-status');
var inversify_1 = require('inversify');
var crypto_1 = require('crypto');
var NON_JSON_TYPES = [String, Number, Boolean], defaultVersion = VersionInformation_1.VersionInformation.create('default', { from: 1 });
var RouteVersion = (function () {
    function RouteVersion(ctrlInstance, ctrlTarget, routeRegistration, middlewares, version) {
        this.ctrlInstance = ctrlInstance;
        this.ctrlTarget = ctrlTarget;
        this.routeRegistration = routeRegistration;
        this.middlewares = middlewares;
        this.version = version;
    }
    return RouteVersion;
}());
var RouteInformation = (function () {
    function RouteInformation(paramHandler, routeUrl, method) {
        this.paramHandler = paramHandler;
        this.routeUrl = routeUrl;
        this.method = method;
        this.versions = [];
        this.wildcardCount = this.routeUrl.split('*').length - 1;
        this.segmentCount = this.routeUrl.split('/').length;
    }
    Object.defineProperty(RouteInformation.prototype, "routeId", {
        get: function () {
            return this.routeUrl + "_" + RouteDecorators_1.RouteMethod[this.method];
        },
        enumerable: true,
        configurable: true
    });
    RouteInformation.prototype.register = function (router) {
        var _this = this;
        if (!this.isUnique()) {
            throw new Errors_1.DuplicateRouteDeclarationError(this.routeUrl, this.method);
        }
        if (this.versions.length === 1 && !this.versions[0].version) {
            var version = this.versions[0];
            this.registerMethod(router, this.method, this.routeUrl, version.middlewares, this.buildRouteMethod(version));
            return;
        }
        var routeVersions = this.versions
            .map(function (version) {
            return {
                url: "/" + _this.getVersionHash(version),
                version: version
            };
        }), blackMagicRouter = express_1.Router({ mergeParams: true });
        blackMagicRouter.use(function (req, res, next) {
            var requestedVersion;
            try {
                requestedVersion = parseInt(req.get(RouteInformation.headerName), 10) || 1;
            }
            catch (e) {
                requestedVersion = 1;
            }
            var requestedRoute = routeVersions.find(function (o) { return o.version.version.isInVersionBounds(requestedVersion); });
            if (!requestedRoute || routeVersions.some(function (o) { return req.url.indexOf(o.url) >= 0; })) {
                return res.status(404).end();
            }
            req.url = requestedRoute.url;
            next();
        });
        for (var _i = 0, routeVersions_1 = routeVersions; _i < routeVersions_1.length; _i++) {
            var version = routeVersions_1[_i];
            this.registerMethod(blackMagicRouter, this.method, version.url, version.version.middlewares, this.buildRouteMethod(version.version));
        }
        router.use(this.routeUrl, blackMagicRouter);
    };
    RouteInformation.prototype.buildRouteMethod = function (version) {
        var _this = this;
        var params = this.paramHandler.getParamsForRoute(version.ctrlTarget.prototype, version.routeRegistration.propertyKey), hasResponseParam = params.some(function (p) { return p.paramType === ParamDecorators_1.ParamType.Response; }), returnType = Reflect.getMetadata('design:returntype', version.ctrlTarget.prototype, version.routeRegistration.propertyKey), ctrlMethod = version.routeRegistration.descriptor.value;
        return function (request, response, next) {
            var errorHandler = Reflect.getMetadata(ErrorHandlerDecorator_1.ERRORHANDLER_KEY, version.ctrlTarget) || new ControllerErrorHandler_1.ControllerErrorHandler(), paramValues = [];
            var handleError = function (error) {
                if (!(error instanceof Error)) {
                    error = new Errors_1.GenericRouteError(_this.routeId, error);
                }
                errorHandler.handleError(version.ctrlInstance, request, response, error);
            };
            try {
                paramValues = _this.paramHandler.getParamValuesForRequest(params, request, response);
                var result = ctrlMethod.apply(version.ctrlInstance, paramValues), responseFunction = function (result) {
                    if (NON_JSON_TYPES.indexOf(result.constructor) !== -1) {
                        response.send(result);
                    }
                    else {
                        response.json(result);
                    }
                };
                if (!returnType && hasResponseParam) {
                    return;
                }
                if (!returnType && !hasResponseParam) {
                    return response.status(httpStatus.NO_CONTENT).end();
                }
                if (!(result instanceof returnType) && !(result.constructor === returnType)) {
                    handleError(new Errors_1.WrongReturnTypeError(version.routeRegistration.propertyKey, returnType, result.constructor));
                }
                if (version.routeRegistration.method === RouteDecorators_1.RouteMethod.Head && returnType === Boolean) {
                    return response.status((result) ? httpStatus.OK : httpStatus.NOT_FOUND).end();
                }
                if (returnType === Promise) {
                    result.then(responseFunction, handleError);
                }
                else {
                    responseFunction(result);
                }
            }
            catch (e) {
                handleError(e);
            }
        };
    };
    RouteInformation.prototype.registerMethod = function (router, httpMethod, url, middlewares, route) {
        switch (httpMethod) {
            case RouteDecorators_1.RouteMethod.Get:
                router.get.apply(router, [url].concat(middlewares, [route]));
                break;
            case RouteDecorators_1.RouteMethod.Put:
                router.put.apply(router, [url].concat(middlewares, [route]));
                break;
            case RouteDecorators_1.RouteMethod.Post:
                router.post.apply(router, [url].concat(middlewares, [route]));
                break;
            case RouteDecorators_1.RouteMethod.Delete:
                router.delete.apply(router, [url].concat(middlewares, [route]));
                break;
            case RouteDecorators_1.RouteMethod.Head:
                router.head.apply(router, [url].concat(middlewares, [route]));
                break;
            default:
                throw new Errors_1.HttpVerbNotSupportedError(httpMethod);
        }
    };
    RouteInformation.prototype.isUnique = function () {
        if (this.versions.length <= 1) {
            return true;
        }
        var _loop_1 = function(version) {
            if (this_1.versions.some(function (o) { return o !== version && RouteHelpers_1.doRouteVersionsOverlap(version.version || defaultVersion, o.version || defaultVersion); })) {
                return { value: false };
            }
        };
        var this_1 = this;
        for (var _i = 0, _a = this.versions; _i < _a.length; _i++) {
            var version = _a[_i];
            var state_1 = _loop_1(version);
            if (typeof state_1 === "object") return state_1.value;
        }
        return true;
    };
    RouteInformation.prototype.getVersionHash = function (version) {
        return crypto_1.createHash('sha256').update(this.routeId + "_" + version.version.toString()).digest('hex');
    };
    return RouteInformation;
}());
var DefaultRouteHandler = (function () {
    function DefaultRouteHandler(paramHandler, config) {
        this.paramHandler = paramHandler;
        this.config = config;
        this.routes = {};
        RouteInformation.headerName = this.config.versionHeaderName;
    }
    DefaultRouteHandler.prototype.addRoutes = function (controllerRegistration, url) {
        var routes = Reflect.getOwnMetadata(RouteDecorators_1.ROUTES_KEY, controllerRegistration.controller) || [], instance = new controllerRegistration.controller(), ctrlVersionInfo = Reflect.getMetadata(VersionDecorator_1.VERSION_KEY, controllerRegistration.controller);
        for (var _i = 0, routes_1 = routes; _i < routes_1.length; _i++) {
            var route = routes_1[_i];
            var routeUrl = url + [controllerRegistration.prefix, route.path].filter(Boolean).join('/'), versionInfo = Reflect.getMetadata(VersionDecorator_1.VERSION_KEY, controllerRegistration.controller, route.propertyKey) || ctrlVersionInfo;
            if (route.method === RouteDecorators_1.RouteMethod.Head && Reflect.getMetadata('design:returntype', controllerRegistration.controller.prototype, route.propertyKey) !== Boolean) {
                throw new Errors_1.HeadHasWrongReturnTypeError();
            }
            var index = void 0;
            if ((index = routeUrl.lastIndexOf('~')) > -1) {
                routeUrl = routeUrl.substring(index + 1);
            }
            if (routeUrl.length > 1) {
                routeUrl = '/' + routeUrl.split('/').filter(Boolean).join('/');
            }
            var routeInfo = new RouteInformation(this.paramHandler, routeUrl, route.method);
            routeInfo = this.routes[routeInfo.routeId] || routeInfo;
            routeInfo.versions.push(new RouteVersion(instance, controllerRegistration.controller, route, controllerRegistration.middlewares.concat(route.middlewares), versionInfo));
            this.routes[routeInfo.routeId] = routeInfo;
        }
    };
    DefaultRouteHandler.prototype.registerRoutes = function (router) {
        var _this = this;
        Object.keys(this.routes)
            .map(function (k) { return _this.routes[k]; })
            .reduce(function (segmentSorted, route) {
            (segmentSorted[route.segmentCount] || (segmentSorted[route.segmentCount] = [])).push(route);
            return segmentSorted;
        }, [])
            .filter(Boolean)
            .reverse()
            .reduce(function (routeList, segments) { return routeList.concat(segments.sort(function (r1, r2) { return r1.wildcardCount - r2.wildcardCount; })); }, [])
            .forEach(function (r) { return r.register(router); });
        return router;
    };
    DefaultRouteHandler.prototype.resetRoutes = function () {
        this.routes = {};
    };
    DefaultRouteHandler = __decorate([
        inversify_1.injectable(),
        __param(0, inversify_1.inject(IoCSymbols_1.IoCSymbols.paramHandler)),
        __param(1, inversify_1.inject(IoCSymbols_1.IoCSymbols.configuration)), 
        __metadata('design:paramtypes', [Object, Object])
    ], DefaultRouteHandler);
    return DefaultRouteHandler;
}());
exports.DefaultRouteHandler = DefaultRouteHandler;
