"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var RouteDecorators_1 = require('../routes/RouteDecorators');
/// Error bases
var DesigntimeError = (function (_super) {
    __extends(DesigntimeError, _super);
    function DesigntimeError() {
        _super.call(this);
    }
    return DesigntimeError;
}(Error));
exports.DesigntimeError = DesigntimeError;
var RuntimeError = (function (_super) {
    __extends(RuntimeError, _super);
    function RuntimeError() {
        _super.call(this);
    }
    return RuntimeError;
}(Error));
exports.RuntimeError = RuntimeError;
var ParameterError = (function (_super) {
    __extends(ParameterError, _super);
    function ParameterError() {
        _super.call(this);
    }
    return ParameterError;
}(RuntimeError));
exports.ParameterError = ParameterError;
var RouteError = (function (_super) {
    __extends(RouteError, _super);
    function RouteError() {
        _super.call(this);
    }
    return RouteError;
}(RuntimeError));
exports.RouteError = RouteError;
var ValidationError = (function (_super) {
    __extends(ValidationError, _super);
    function ValidationError() {
        _super.call(this);
    }
    return ValidationError;
}(ParameterError));
exports.ValidationError = ValidationError;
/// Registration & Controller errors
/**
 * Error that is thrown when a http method is not supported by the framework.
 * (Error will happen at startup)
 *
 * @class HttpVerbNotSupportedError
 * @extends {DesigntimeError}
 */
var HttpVerbNotSupportedError = (function (_super) {
    __extends(HttpVerbNotSupportedError, _super);
    function HttpVerbNotSupportedError(method) {
        _super.call(this);
        this.message = "HttpVerb not supported; " + method;
    }
    return HttpVerbNotSupportedError;
}(DesigntimeError));
exports.HttpVerbNotSupportedError = HttpVerbNotSupportedError;
/**
 * Error that is thrown when a route is duplicated (same url and method).
 * (Error will happen at startup)
 *
 * @class DuplicateRouteDeclarationError
 * @extends {DesigntimeError}
 */
var DuplicateRouteDeclarationError = (function (_super) {
    __extends(DuplicateRouteDeclarationError, _super);
    function DuplicateRouteDeclarationError(url, method) {
        _super.call(this);
        this.message = "The route to url \"" + url + "\" with http method \"" + RouteDecorators_1.RouteMethod[method] + "\" is declared twice.\nIf you use versioning, the versions could overlap!";
    }
    return DuplicateRouteDeclarationError;
}(DesigntimeError));
exports.DuplicateRouteDeclarationError = DuplicateRouteDeclarationError;
/**
 * Error that is thrown when a head route has a non boolean return type.
 * (Error will happen at startup)
 *
 * @class HeadHasWrongReturnTypeError
 * @extends {DesigntimeError}
 */
var HeadHasWrongReturnTypeError = (function (_super) {
    __extends(HeadHasWrongReturnTypeError, _super);
    function HeadHasWrongReturnTypeError() {
        _super.call(this);
        this.message = 'Head route must have return type boolean';
    }
    return HeadHasWrongReturnTypeError;
}(DesigntimeError));
exports.HeadHasWrongReturnTypeError = HeadHasWrongReturnTypeError;
/**
 * Error that is thrown when an error handler accepts the wrong arguments.
 * (Error will happen at startup)
 *
 * @class ErrorHandlerWrongArgumentsError
 * @extends {DesigntimeError}
 */
var ErrorHandlerWrongArgumentsError = (function (_super) {
    __extends(ErrorHandlerWrongArgumentsError, _super);
    function ErrorHandlerWrongArgumentsError() {
        _super.call(this);
        this.message = 'Error handler must accept exactly request, response and an error object';
    }
    return ErrorHandlerWrongArgumentsError;
}(DesigntimeError));
exports.ErrorHandlerWrongArgumentsError = ErrorHandlerWrongArgumentsError;
/**
 * Error that is thrown when an error handler accepts the wrong argument types.
 * (Error will happen at startup)
 *
 * @class ErrorHandlerWrongArgumentTypesError
 * @extends {DesigntimeError}
 */
var ErrorHandlerWrongArgumentTypesError = (function (_super) {
    __extends(ErrorHandlerWrongArgumentTypesError, _super);
    function ErrorHandlerWrongArgumentTypesError() {
        _super.call(this);
        this.message = 'Error handler arguments must be: Object, Object, Error';
    }
    return ErrorHandlerWrongArgumentTypesError;
}(DesigntimeError));
exports.ErrorHandlerWrongArgumentTypesError = ErrorHandlerWrongArgumentTypesError;
/**
 * Error that is thrown when an error handler has a non void return type.
 * (Error will happen at startup)
 *
 * @class ErrorHandlerWrongReturnTypeError
 * @extends {DesigntimeError}
 */
var ErrorHandlerWrongReturnTypeError = (function (_super) {
    __extends(ErrorHandlerWrongReturnTypeError, _super);
    function ErrorHandlerWrongReturnTypeError() {
        _super.call(this);
        this.message = 'Error handler must have return type void';
    }
    return ErrorHandlerWrongReturnTypeError;
}(DesigntimeError));
exports.ErrorHandlerWrongReturnTypeError = ErrorHandlerWrongReturnTypeError;
/// Parameter & Parsing errors
/**
 * Error that is thrown when a given parameter type supports not at least argument (used for instantiation of the type).
 * (Error will happen at startup)
 *
 * @class ParameterParseError
 * @extends {DesigntimeError}
 */
var ParameterConstructorArgumentsError = (function (_super) {
    __extends(ParameterConstructorArgumentsError, _super);
    function ParameterConstructorArgumentsError(name) {
        _super.call(this);
        this.message = "The constructor for the parameter \"" + name + "\" must accept at least 1 argument";
    }
    return ParameterConstructorArgumentsError;
}(DesigntimeError));
exports.ParameterConstructorArgumentsError = ParameterConstructorArgumentsError;
/**
 * Error that is thrown when a required parameter is not provided. Url parameters are always required.
 * (Runtime error)
 *
 * @class RequiredParameterNotProvidedError
 * @extends {ParameterError}
 */
var RequiredParameterNotProvidedError = (function (_super) {
    __extends(RequiredParameterNotProvidedError, _super);
    function RequiredParameterNotProvidedError(name) {
        _super.call(this);
        this.message = "The required parameter \"" + name + "\" was not provided";
    }
    return RequiredParameterNotProvidedError;
}(ParameterError));
exports.RequiredParameterNotProvidedError = RequiredParameterNotProvidedError;
/**
 * Error that is thrown when the parsing process of a parameters throws an error.
 * (Runtime error)
 *
 * @class ParameterParseError
 * @extends {ParameterError}
 */
var ParameterParseError = (function (_super) {
    __extends(ParameterParseError, _super);
    function ParameterParseError(name, innerException) {
        _super.call(this);
        this.innerException = innerException;
        this.message = "Parsing of the parameter \"" + name + "\" threw an error.\nInnerException: " + innerException;
    }
    return ParameterParseError;
}(ParameterError));
exports.ParameterParseError = ParameterParseError;
/// Route errors
/**
 * Error that is thrown when the route method returns the wrong type (i.e. a string instead of a number).
 * (Runtime error)

 * @class WrongReturnTypeError
 * @extends {RouteError}
 */
var WrongReturnTypeError = (function (_super) {
    __extends(WrongReturnTypeError, _super);
    function WrongReturnTypeError(name, expected, received) {
        _super.call(this);
        this.message = "The method \"" + name + "\" returned the wrong result type.\nExpected: " + expected + "\nReceived: " + received;
    }
    return WrongReturnTypeError;
}(RouteError));
exports.WrongReturnTypeError = WrongReturnTypeError;
/**
 * Error that is thrown when the route method throws any error (or any Promise returned by a route).
 * (Runtime error)
 *
 * @class GenericRouteError
 * @extends {RouteError}
 */
var GenericRouteError = (function (_super) {
    __extends(GenericRouteError, _super);
    function GenericRouteError(name, innerException) {
        _super.call(this);
        this.innerException = innerException;
        this.message = "The method \"" + name + "\" threw an error.\nInnerException: " + innerException;
    }
    return GenericRouteError;
}(RouteError));
exports.GenericRouteError = GenericRouteError;
/// Validator errors
/**
 * Error that is thrown when the - if provided - validator of a parameter returns "false".
 * (Runtime error)
 *
 * @class ParamValidationFailedError
 * @extends {ValidationError}
 */
var ParamValidationFailedError = (function (_super) {
    __extends(ParamValidationFailedError, _super);
    function ParamValidationFailedError(name) {
        _super.call(this);
        this.message = "The validator for the parameter \"" + name + "\" was not valid.";
    }
    return ParamValidationFailedError;
}(ValidationError));
exports.ParamValidationFailedError = ParamValidationFailedError;
/// Versioning errors
/**
 * Error that is thrown when a version information on a route or a controller is missing both information,
 * i.e. there is no version in "from" and no version in "until"
 * (Error will happen at startup)
 *
 * @class VersionInformationMissing
 * @extends {DesigntimeError}
 */
var VersionInformationMissing = (function (_super) {
    __extends(VersionInformationMissing, _super);
    function VersionInformationMissing(ctrlOrRouteName) {
        _super.call(this);
        this.message = "The controller or method \"" + ctrlOrRouteName + "\" has neither from nor until version information.";
    }
    return VersionInformationMissing;
}(DesigntimeError));
exports.VersionInformationMissing = VersionInformationMissing;
/**
 * Error that is thrown when a version information contains errors.
 * (Error will happen at startup)
 *
 * @class VersionInformationInvalid
 * @extends {DesigntimeError}
 */
var VersionInformationInvalid = (function (_super) {
    __extends(VersionInformationInvalid, _super);
    function VersionInformationInvalid(ctrlOrRouteName, reason) {
        _super.call(this);
        this.message = "The controller or method \"" + ctrlOrRouteName + "\" has invalid version information.\nReason: " + reason;
    }
    return VersionInformationInvalid;
}(DesigntimeError));
exports.VersionInformationInvalid = VersionInformationInvalid;
/**
 * Error that is thrown when a route or a controller contains multiple
 * version information.
 * (Error will happen at startup)
 *
 * @class DuplicateVersionInformation
 * @extends {DesigntimeError}
 */
var DuplicateVersionInformation = (function (_super) {
    __extends(DuplicateVersionInformation, _super);
    function DuplicateVersionInformation(ctrlOrRouteName) {
        _super.call(this);
        this.message = "The controller or method \"" + ctrlOrRouteName + "\" has duplicated version decorators.";
    }
    return DuplicateVersionInformation;
}(DesigntimeError));
exports.DuplicateVersionInformation = DuplicateVersionInformation;
