"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
var IoC_1 = require('./core/IoC');
var IoCSymbols_1 = require('./core/IoCSymbols');
__export(require('./controllers/ControllerDecorator'));
__export(require('./errors/ErrorHandlerDecorator'));
__export(require('./errors/ControllerErrorHandler'));
__export(require('./errors/Errors'));
__export(require('./params/ParamDecorators'));
__export(require('./routes/RouteDecorators'));
__export(require('./validators/Validators'));
__export(require('./versioning/VersionDecorator'));
var registrar = IoC_1.IocContainer.get(IoCSymbols_1.IoCSymbols.registrar);
/**
 * Function that loads and registers all controllers from a given directory. All found files are "required" and
 * should not throw any errors, or else the promise is rejected.
 *
 * @param {string} folderPath - The root path to start the search for the controllers.
 * @param {string} [root=process.cwd()] - The root of the project. Can be different if you start your app through a script.
 * @param {boolean} [recursive=false] - Defines if the function searches recursively for controllers.
 * @param {RegExp} [matchRegExp=/(.*)[.]js$/] - The regular expression that must be mached before a file is required.
 * @param {string} [baseUrl=''] - Base url for the routing system. Will be prefixed for all controllers.
 * @param {Router} [router=Router()] - Express router to attach the routes to. If omitted, a new router is instantiated.
 * @returns {Promise<Router>} - A promise that resolves with the configured router instance. Or rejects when an error happens.
 */
exports.registerControllersFromFolder = registrar.registerControllersFromFolder.bind(registrar);
/**
 * Function that registers all decorated controller with their decorated route functions in an expressJS router.
 * When no router is provided, this function will instantiate and return a router.
 *
 * @param {string} [baseUrl=''] - Base url for the routing system. Will be prefixed for all controllers.
 * @param {Router} [router=Router()] - Express router to attach the routes to. If omitted, a new router is instantiated.
 * @returns {Router} - The configured router.
 */
exports.registerControllers = registrar.registerControllers.bind(registrar);
/**
 * Injected object that contains the global configuration for giuseppe.
 * May be changed before the registration process (e.g. accept header version setting).
 */
exports.giuseppeConfiguration = IoC_1.IocContainer.get(IoCSymbols_1.IoCSymbols.configuration);
