"use strict";
var Errors_1 = require('../errors/Errors');
function isDefined(value) {
    return value !== null && value !== undefined;
}
function isInvalid(value) {
    return isDefined(value) && (value.constructor !== Number || value < 1 || value % 1 !== 0);
}
/**
 * Defines version information for a controller or a route. Throws if there are errors with the information.
 *
 * @class VersionInformation
 */
var VersionInformation = (function () {
    function VersionInformation(from, until) {
        this.from = from;
        this.until = until;
    }
    VersionInformation.create = function (name, versionInformation) {
        if (!(isDefined(versionInformation.from) || isDefined(versionInformation.until))) {
            throw new Errors_1.VersionInformationMissing(name);
        }
        if (isInvalid(versionInformation.from)) {
            throw new Errors_1.VersionInformationInvalid(name, "The from value (" + versionInformation.from + ") is either not a number, a floating point number or less than 1");
        }
        if (isInvalid(versionInformation.until)) {
            throw new Errors_1.VersionInformationInvalid(name, "The until value (" + versionInformation.until + ") is either not a number, a floating point number or less than 1");
        }
        if (isDefined(versionInformation.from) && isDefined(versionInformation.until) && versionInformation.from > versionInformation.until) {
            throw new Errors_1.VersionInformationInvalid(name, "The from value (" + versionInformation.from + ") is greater than the until (" + versionInformation.until + ")");
        }
        return new VersionInformation(versionInformation.from || -Infinity, versionInformation.until || Infinity);
    };
    VersionInformation.prototype.isInVersionBounds = function (requestedVersion) {
        return this.from <= requestedVersion && requestedVersion <= this.until;
    };
    VersionInformation.prototype.toString = function () {
        return this.from.toString() + "|" + this.until.toString();
    };
    return VersionInformation;
}());
exports.VersionInformation = VersionInformation;
