/// <reference types="express-serve-static-core" />
import 'reflect-metadata';
import { RequestHandler } from 'express-serve-static-core';
/**
 * Reflect metadata key for the controllers routes.
 *
 * @type {string}
 */
export declare const ROUTES_KEY: string;
/**
 * Enum for the possible route http methods.
 */
export declare enum RouteMethod {
    Get = 0,
    Put = 1,
    Post = 2,
    Delete = 3,
    Head = 4,
}
/**
 * Declares the given method as an api route. Adds the route registration to the controller.
 * All route registrations are decorated and registered during the registerControllers method.
 *
 * @param {string} [route=''] - The routed endpoint of the method. If omitted, the base route is taken.
 * @param {RouteMethod} httpMethod - The http verb to use for this route.
 * @param {RequestHandler[]} [middlewares=[]] - The used middleware(s) for this route.
 * @returns {(any, string, PropertyDescriptor) => void} - Method decorator for the given function.
 */
export declare function Route(route?: string, httpMethod?: RouteMethod, ...middlewares: RequestHandler[]): (target: any, propertyKey: string, descriptor: PropertyDescriptor) => void;
/**
 * Alias function for @Route(string, RouteMethod.Get)
 *
 * @param {string} [route=''] - The routed endpoint of the method. If omitted, the base route is taken.
 * @param {RequestHandler[]} [middlewares=[]] - The used middleware(s) for this route.
 * @returns {(any, string, PropertyDescriptor) => void} - Method decorator for the given function with the http verb 'GET'.
 */
export declare function Get(route?: string, ...middlewares: RequestHandler[]): (target: any, propertyKey: string, descriptor: PropertyDescriptor) => void;
/**
 * Alias function for @Route(string, RouteMethod.Put)
 *
 * @param {string} [route=''] - The routed endpoint of the method. If omitted, the base route is taken.
 * @param {RequestHandler[]} [middlewares=[]] - The used middleware(s) for this route.
 * @returns {(any, string, PropertyDescriptor) => void} - Method decorator for the given function with the http verb 'PUT'.
 */
export declare function Put(route?: string, ...middlewares: RequestHandler[]): (target: any, propertyKey: string, descriptor: PropertyDescriptor) => void;
/**
 * Alias function for @Route(string, RouteMethod.Post)
 *
 * @param {string} [route=''] - The routed endpoint of the method. If omitted, the base route is taken.
 * @param {RequestHandler[]} [middlewares=[]] - The used middleware(s) for this route.
 * @returns {(any, string, PropertyDescriptor) => void} - Method decorator for the given function with the http verb 'POST'.
 */
export declare function Post(route?: string, ...middlewares: RequestHandler[]): (target: any, propertyKey: string, descriptor: PropertyDescriptor) => void;
/**
 * Alias function for @Route(string, RouteMethod.Delete)
 *
 * @param {string} [route=''] - The routed endpoint of the method. If omitted, the base route is taken.
 * @param {RequestHandler[]} [middlewares=[]] - The used middleware(s) for this route.
 * @returns {(any, string, PropertyDescriptor) => void} - Method decorator for the given function with the http verb 'DELETE'.
 */
export declare function Delete(route?: string, ...middlewares: RequestHandler[]): (target: any, propertyKey: string, descriptor: PropertyDescriptor) => void;
/**
 * Alias function for @Route(string, RouteMethod.Head)
 *
 * @param {string} [route=''] - The routed endpoint of the method. If omitted, the base route is taken.
 * @param {RequestHandler[]} [middlewares=[]] - The used middleware(s) for this route.
 * @returns {(any, string, PropertyDescriptor) => void} - Method decorator for the given function with the http verb 'Head'.
 */
export declare function Head(route?: string, ...middlewares: RequestHandler[]): (target: any, propertyKey: string, descriptor: PropertyDescriptor) => void;
