/**
 * Reflect metadata key for the version information.
 *
 * @type {string}
 */
export declare const VERSION_KEY: string;
/**
 * Version decorator; decorates a class or a method of a class as versioned.
 * When the routes are registered, the version information will determine
 * if a route is accessable or not. If a route is not available in a certain version
 * the system should return a 404.
 *
 * @param {{from: ?number, to: ?number}} versionInformation - An object with from and / or until information. If both are omitted, an error is thrown.
 * @returns {(Function|Object, string?, PropertyDescriptor?) => void} - Decorator for a versioned controller or route.
 */
export declare function Version(versionInformation: {
    from?: number;
    until?: number;
}): (controllerOrRoute: any, propertyKey?: string, descriptor?: PropertyDescriptor) => void;
